using System;
using System.Collections.Generic;
using  DataContracts;
namespace Ato.EN.IntegrationServices.CodeGenerationCTR
{
    public partial class CTR2021Validator
    {

        #region Manual Rules - Rules marked for manual coding

        #region VR.ATO.CTR.500203

        /*  VR.ATO.CTR.500203
                    If Tax rate 2019-2020 is provided, then a valid range for Tax rate 2019-2020 is 0.0001 - 0.45000

                Legacy Rule Format:
                    IF [CTR409] <> NULL AND ([CTR409] <> SET(0.0001- 0.45000))
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^CTR409 <> NULL AND (NotInSet(^CTR409, '0.0001-0.45000'))

        Data Elements:
    
            ^CTR409 = RP.Y0-1:TaxRate.Designation.Rate
            */
        protected void VRATOCTR500203(List<ProcessMessageDocument> response, CTR2021 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            assertion = (report.CTR409 != null && (report.CTR409.GetValueOrDefault() < 0.0001M || report.CTR409.GetValueOrDefault() > 0.45000M));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.500203",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Valid range for Tax rate 2019-2020 is 0.0001 - 0.45000",
                    Location = "/xbrli:xbrl/tns:TaxRate.Designation.Rate",//[@contextRef='" + report.RPY0Minus1Id + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.CTR.500203" } },
                };

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "CTR409", Value = GetValueOrEmpty(report.CTR409) });

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.500203

        #region VR.ATO.CTR.500204

        /*  VR.ATO.CTR.500204
                    If Tax rate 2020-2021 is provided, then a valid range for Tax rate 2020-2021 is 0.0001 - 0.45000

                Legacy Rule Format:
                    IF [CTR410] <> NULL AND ([CTR410] <> SET(0.0001,0.45000))
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^CTR410 <> NULL AND (NotInSet(^CTR410, '0.0001,0.45000'))

        Data Elements:
    
            ^CTR410 = RP:TaxRate.Designation.Rate
            */
        protected void VRATOCTR500204(List<ProcessMessageDocument> response, CTR2021 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            assertion = (report.CTR410 != null && (report.CTR410.GetValueOrDefault() < 0.0001M || report.CTR410.GetValueOrDefault() > 0.45000M));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.500204",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Valid range for Tax rate 2020-2021 is 0.0001 - 0.45000",
                    Location = "/xbrli:xbrl/tns:TaxRate.Designation.Rate",//[@contextRef='" + report.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.CTR.500204" } },
                };

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "CTR410", Value = GetValueOrEmpty(report.CTR410) });



                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.500204
        #endregion Manual Rules - Rules marked for manual coding

    }
}

