using System;
using System.Collections.Immutable;
using Ato.CD.Inbound.Shared;
using Ato.EN.IntegrationServices.CodeGenerationTRT;
using DataContracts;

namespace Ato.CD.Inbound.TRT202102
{
    internal class CrossFormLocator
    {
        private static readonly ImmutableDictionary<string, Func<TRT2021, BusinessDocument, ICrossFormValidator>> CrossFormValidators;

        static CrossFormLocator()
        {
            var builder = ImmutableDictionary.CreateBuilder<string, Func<TRT2021, BusinessDocument, ICrossFormValidator>>();

            builder.Add(Schedule.CGTS.ToString(), (report, schedule) => new CrossFormValidatorCGTS(report, schedule));
            builder.Add(Schedule.LS.ToString(), (report, schedule) => new CrossFormValidatorLS(report, schedule));
            builder.Add(Schedule.PSS.ToString(), (report, schedule) => new CrossFormValidatorPSS(report, schedule));
            builder.Add(Schedule.FTER.ToString(), (report, schedule) => new CrossFormValidatorFTER(report, schedule));
            builder.Add(Schedule.IEE.ToString(), (report, schedule) => new CrossFormValidatorIEE(report, schedule));
            builder.Add(Schedule.RS.ToString(), (report, schedule) => new CrossFormValidatorRS(report, schedule));
            builder.Add(Schedule.TRTAMIS.ToString(), (report, schedule) => new CrossFormValidatorTRTAMIS(report, schedule));
            builder.Add(Schedule.IDS.ToString(), (report, schedule) => new CrossFormValidatorIDS(report, schedule));

            CrossFormValidators = builder.ToImmutable();
        }

        internal static ICrossFormValidator LocateCrossFormValidatorFor(BusinessDocument schedule, TRT2021 report)
            => CrossFormValidators[schedule.DocumentName](report, schedule);
    }
}